package id.co.sangkuriang.inovasi.daerah.master.interceptor

import com.auth0.jwt.JWT
import com.auth0.jwt.algorithms.Algorithm
import com.auth0.jwt.exceptions.JWTCreationException
import com.auth0.jwt.exceptions.JWTDecodeException
import com.auth0.jwt.exceptions.JWTVerificationException
import com.fasterxml.jackson.databind.ObjectMapper
import id.co.sangkuriang.inovasi.daerah.master.constant.HEADER_STRING
import id.co.sangkuriang.inovasi.daerah.master.constant.SECRET
import id.co.sangkuriang.inovasi.daerah.master.constant.TOKEN_PREFIX
import id.co.sangkuriang.inovasi.daerah.master.model.response.ReturnData
import id.co.sangkuriang.inovasi.daerah.master.utility.getPathUrl
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.core.GrantedAuthority
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.web.util.ContentCachingRequestWrapper
import javax.servlet.Filter
import javax.servlet.FilterChain
import javax.servlet.ServletRequest
import javax.servlet.ServletResponse
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse


class TokenInterceptor : Filter {

    override fun doFilter(request: ServletRequest?, response: ServletResponse?, chain: FilterChain?) {
        val req = ContentCachingRequestWrapper(request as HttpServletRequest)
        val res = response as HttpServletResponse

        // Read the Authorization header, where the JWT token should be
        val header = req.getHeader(HEADER_STRING)

        // If header doesn't contain Bearer or is null will return nothing and exit
        if (header == null || !header.startsWith(TOKEN_PREFIX)) {
            responseException(res, "Sorry, your provided token information not exists")
            return
        }


        var authentication: UsernamePasswordAuthenticationToken? = null
        var successAuth = true

        try {
            authentication = getAuthentication(req)
        } catch (e: JWTDecodeException) {
            successAuth = false
            responseException(res, e.message.toString())
        } catch (e: JWTCreationException) {
            successAuth = false
            responseException(res, e.message.toString())
        } catch (e: JWTVerificationException) {
            successAuth = false
            responseException(res, e.message.toString())
        } catch (e: IllegalArgumentException) {
            successAuth = false
            responseException(res, e.message.toString())
        }

        SecurityContextHolder.getContext().authentication = authentication
        if (successAuth) {
            chain?.doFilter(request, response)
        }
    }

    private fun responseException(response: HttpServletResponse, message: String) {
        val json = ObjectMapper().writeValueAsString(
            ReturnData(
                message = message
            )
        )

        response.contentType = "application/json"
        response.status = HttpServletResponse.SC_UNAUTHORIZED
        response.writer.write(json)
        response.writer.flush()
        response.writer.close()
    }

    @Throws(
        JWTDecodeException::class,
        JWTCreationException::class,
        JWTVerificationException::class,
        IllegalArgumentException::class
    )

    private fun getAuthentication(request: HttpServletRequest): UsernamePasswordAuthenticationToken? {
        val token = request.getHeader(HEADER_STRING)
        if (token != null) {
            // parse jwt token from request and validate it internal authentication
            val jwt =
                JWT.require(Algorithm.HMAC512(SECRET.toByteArray())).build().verify(token.replace(TOKEN_PREFIX, ""))
            return if (jwt.subject != null) { // return principal username
                val pathUrl = getPathUrl(request)
                val audience = jwt.audience?.get(0)

                println("pathUrl => $audience - ${request.method} - $pathUrl")

                if (audience == "user") {
                    val tokenParse = token + System.currentTimeMillis()
                    JWT.require(Algorithm.HMAC512(SECRET.toByteArray())).build()
                        .verify(tokenParse.replace(TOKEN_PREFIX, ""))
                }

                UsernamePasswordAuthenticationToken(
                    jwt.subject,
                    audience,
                    ArrayList<GrantedAuthority>()
                )
            } else {
                null
            }
        }
        return null
    }
}
